use std::io;
use serde::Serialize;
use strum_macros::{AsRefStr, Display, EnumString};
use thiserror::Error;
use crate::core::packet::{Packet, PacketError};

// #[allow(unused_variables, dead_code)]
#[derive(Error, Debug)]
pub enum MessageError {
    #[error("{0}")]
    InvalidPacket(#[from] PacketError),
    #[error("no message available")]
    NoMessage,
    #[error("unknown message error")]
    Unknown,
}

/// 命令名对照表，用于对应解析命令名称
// #[allow(unused_variables, dead_code)]
#[derive(Debug, PartialEq, Display, AsRefStr, EnumString)]
pub enum Command {
    Undefined,
    #[strum(serialize = "DANMU_MSG")]
    DanmuMsg,
    #[strum(serialize = "SUPER_CHAT_MESSAGE")]
    SuperChatMessage,
    #[strum(serialize = "SUPER_CHAT_MESSAGE_DELETE")]
    SuperChatMessageDelete,
    #[strum(serialize = "SEND_GIFT")]
    SendGift,
}

impl Default for Command {
    fn default() -> Self {
        Self::Undefined
    }
}

impl Command {
    pub fn from_str(cmd: &str) -> Self {
        cmd.parse().unwrap_or_default()
    }
}

#[allow(unused_variables, dead_code)]
#[derive(Serialize, Debug)]
pub enum Message {
    DanmuMsg {},
    SuperChatMessage,
    SuperChatMessageDelete,
}

impl Message {
    fn deserialize_json() {}

    fn parse_packet_skip_err(single_packet: &Packet) -> Message {}

    /// 从 Packet 中提取 Message
    ///
    /// 只有 Json 格式的非业务消息会被提取，即心跳包回复，认证请求回复等不会被解析。
    pub fn from_packet(packet: &mut Packet) -> Result<Vec<Self>, MessageError> {
        let mut packets = packet.get_body_packets()?;
        if packets.len() == 0 {
            return Err(MessageError::NoMessage);
        };

        let messages = packets.iter().map(|packet| {
            Self::parse_packet_skip_err(packet)
        }).collect();
        if messages.len() == 0 {
            Err(MessageError::NoMessage)
        } else {
            Ok(messages)
        }
    }
}